package members

import (
	"fmt"
	"testing"

	"github.com/stretchr/testify/require"
)

func TestFormat(t *testing.T) {
	tests := []struct {
		name     string
		template string
		want     string
		role     Role
	}{
		{
			name:     "NotMember with %s",
			template: "%s",
			role:     NotMember,
			want:     "not member",
		},
		{
			name:     "RoleMember with %s",
			role:     RoleMember,
			template: "%s",
			want:     "member",
		},
		{
			name:     "RoleOwner with %s",
			role:     RoleOwner,
			template: "%s",
			want:     "owner",
		},
		{
			name:     "RoleAdmin with %s",
			role:     RoleAdmin,
			template: "%s",
			want:     "admin",
		},
		{
			name:     "Existent role with %d",
			role:     RoleAdmin,
			template: "%d",
			want:     "3",
		},
		{
			name:     "Non-existent role with %s",
			role:     4,
			template: "%s",
			want:     "4",
		},
		{
			name:     "Non-existent role with %d",
			role:     4,
			template: "%d",
			want:     "4",
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			require.Equal(t, tt.want, fmt.Sprintf(tt.template, tt.role))
		})
	}
}
